unit Dhrystone;

interface

	var
		NOM, NOE: integer;
		AVG, MIN, MAX: real;

	procedure doDhrystone;

implementation

	const
		NumberOfExecutions = 10000;
		NumberOfMeasurements = 10;
		LargeRealNumber = 1000000.0;
		MicrosecondsPerClock = 16667;

	type
		Enumeration = (Ident1, Ident2, Ident3, Ident4, Ident5);

		OneToThirty = 1..30;
		OneToFifty = 1..50;
		CapitalLetter = 'A'..'Z';

		String30 = packed array[1..30] of char;

		Array1DimInteger = array[OneToFifty] of integer;
		Array2DimInteger = array[OneToFifty, OneToFifty] of integer;

		RecordPointer = ^RecordType;

		RecordType = record
				PointerComp: RecordPointer;
				case Discr : Enumeration of


					Ident1: (
							EnumComp: Enumeration;
							IntComp: OneToFifty;
							StringComp: String30
					);
					Ident2: (
							EnumComp2: Enumeration;
							StringComp2: String30
					);
					Ident3, Ident4, Ident5: (
							CharComp1, CharComp2: char
					);
			end;  (*record*)

	procedure doDhrystone;

		var
			ExecutionIndex: 1..NumberOfExecutions;
			MeasurementIndex: 1..NumberOfMeasurements;
			BeginClock, EndClock: longint;
			SumClocks, EmptyLoopClocks, TimePerExecution, SumTime, MinTime, MaxTime: real;
			TotalTime: real;

			PointerGlob, NextPointerGlob: RecordPointer;
			IntGlob: integer;

			BoolGlob: boolean;
			CharGlob1, CharGlob2: char;
			ArrayGlob1: Array1DimInteger;
			ArrayGlob2: Array2DimInteger;

			IntGlob1, IntGlob2, IntGlob3: OneToFifty;
			CharIndex: char;
			EnumGlob: Enumeration;
			StringGlob1, StringGlob2: String30;

		procedure Proc1 (PointerParVal: RecordPointer);
		forward;
		procedure Proc2 (var IntParRef: OneToFifty);
		forward;
		procedure Proc3 (var PointerParRef: RecordPointer);
		forward;
		procedure Proc4;
		forward;
		procedure Proc5;
		forward;
		procedure Proc6 (EnumParVal: Enumeration;
										var EnumParRef: Enumeration);
		forward;
		procedure Proc7 (IntPar1Val, IntPar2Val: OneToFifty;
										var IntParRef: OneToFifty);
		forward;
		procedure Proc8 (var ArrayPar1Ref: Array1DimInteger;
										var ArrayPar2Ref: Array2DimInteger;
										IntPar1Val, IntPar2Val: integer);
		forward;
		function Func1 (CharPar1Val, CharPar2Val: CapitalLetter): Enumeration;
		forward;
		function Func2 (var StringPar1Ref, StringPar2Ref: String30): boolean;
		forward;
		function Func3 (EnumParVal: Enumeration): boolean;
		forward;

		procedure Proc1; (* (PointerParVal: RecordPointer) *)

		begin
			with PointerParVal^.PointerComp^ do
				begin
					PointerParVal^.PointerComp^ := PointerGlob^;
					PointerParVal^.IntComp := 5;
					IntComp := PointerParVal^.IntComp;
					PointerComp := PointerParVal^.PointerComp;
					Proc3(PointerComp);
					if Discr = Ident1 then
						begin
							IntComp := 6;
							Proc6(PointerParVal^.EnumComp, EnumComp);
							PointerComp := PointerGlob^.PointerComp;
							Proc7(IntComp, 10, IntComp);
						end (*then *)
					else (* not executed *)
						PointerParVal^ := PointerParVal^.PointerComp^;
				end; (* with*)
		end; (* Proc1*)

		procedure Proc2; (* (var IntParRef: OneToFifty) *)

(* executed once*)
			var
				IntLoc: OneToFifty;
				EnumLoc: Enumeration;

		begin
			IntLoc := IntParRef + 10;
			repeat (* executed once *)
				if CharGlob1 = 'A' then (* executed *)
					begin
						IntLoc := IntLoc - 1;
						IntParRef := IntLoc - IntGlob;
						EnumLoc := Ident1;
					end (* if *)
			until EnumLoc = Ident1; (* true *)
		end; (* Proc2 *)

		procedure Proc3; (* (var PointerParRef: RecordPointer) *)

 (* executed once *)
		begin
			if PointerGlob <> nil then (* executed *)
				PointerParRef := PointerGlob^.PointerComp
			else (* not executed *)
				IntGlob := 100;
{ Proc7 (10, IntGlob, PointerGlob^.IntComp);}
		end; (* Proc3 *)

		procedure Proc4; (* executed once *)

			var
				BoolLoc: boolean;

		begin
			BoolLoc := CharGlob1 = 'A';
			BoolLoc := BoolLoc or BoolGlob;
			CharGlob2 := 'B';
		end; (* Proc4 *)

		procedure Proc5; (* executed once *)

		begin
			CharGlob1 := 'A';
			BoolGlob := false;
		end; (* Proc5 *)

		procedure Proc6; (* ( EnumParVal: Enumeration;*)
       (*var EnumParRef: Enumeration) *)
 (* executed once *)

		begin
			EnumParRef := EnumParVal;
			if not Func3(EnumParVal) then (* not executed *)
				EnumParRef := Ident4;
			case EnumParVal of
				Ident1: 
					EnumParRef := Ident1;
				Ident2: 
					if IntGlob > 100 then
						EnumParRef := Ident1
					else
						EnumParRef := Ident4;
				Ident3: 
					EnumParRef := Ident2; (* executed *)
				Ident4: 
					;
				Ident5: 
					EnumParRef := Ident3;
			end; (* case *)
		end; (* Proc6 *)

		procedure Proc7; (* ( IntPar1Val,IntPar2Val: OneToFifty;*)
            (*var IntParRef: OneToFifty) *)

(* executed three times *)
			var
				IntLoc: OneToFifty;

		begin
			IntLoc := IntPar1Val + 2;
			IntParRef := IntPar2Val + IntLoc;
		end; (* Proc7 *)

		procedure Proc8; (* (var ArrayPar1Ref: Array1DimInteger;*)
            (* var ArrayPar2Ref: Array2DimInteger;*)
            (*IntPar1Val,IntPar2Val : integer) *)
(* executed once *)

			var
				IntIndex, IntLoc: OneToFifty;

		begin
			IntLoc := IntPar1Val + 5;
			ArrayPar1Ref[IntLoc] := IntPar2Val;
			ArrayPar1Ref[IntLoc + 1] := ArrayPar1Ref[IntLoc];
			ArrayPar1Ref[IntLoc + 30] := IntLoc;
			for IntIndex := IntLoc to IntLoc + 1 do
				ArrayPar2Ref[IntLoc, IntIndex] := IntLoc;
			ArrayPar2Ref[IntLoc, IntLoc - 1] := ArrayPar2Ref[IntLoc, IntLoc - 1] + 1;
			ArrayPar2Ref[IntLoc + 20, IntLoc] := ArrayPar1Ref[IntLoc];
			IntGlob := 5;
		end;  (* Proc8 *)

		function Func1; (* (CharPar1Val,CharPar2Val: CapitalLetter): Enumeration *)
(* executed 3 times, always returns Ident1 *)

			var
				CharLoc1, CharLoc2: CapitalLetter;

		begin
			CharLoc1 := CharPar1Val;
			CharLoc2 := CharLoc1;
			if CharLoc2 <> CharPar2Val then (* executed *)
				Func1 := Ident1
			else (* not executed *)
				Func1 := Ident2;
		end; (* Func1 *)

		function Func2; (* ( var StringPar1Ref,StringPar2Ref: String30): boolean *)
(* executed once, returns false *)

			var
				IntLoc: OneToThirty;
				CharLoc: CapitalLetter;

		begin
			IntLoc := 2;
			while IntLoc <= 2 do (* body executed once *)
				if Func1(StringPar1Ref[IntLoc], StringPar2Ref[IntLoc + 1]) = Ident1 then (* executed *)
					begin
						CharLoc := 'A';
						IntLoc := IntLoc + 1;
					end; (* if, while *)
			if (CharLoc >= 'W') and (CharLoc < 'Z') then (* not executed *)
				IntLoc := 7;
			if CharLoc = 'X' then (* not executed *)
				func2 := true
			else (* executed *)
				begin
					if StringPar1Ref > StringPar2Ref then (* not executed *)
						begin
							IntLoc := IntLoc + 7;
							Func2 := true
						end
					else (* executed *)
						Func2 := false;
				end; (* if CharLoc *)
		end; (* Func2 *)

		function Func3; (* (EnumParVal: Enumeration): boolean *)
 (* executed once, returns true *)

			var
				EnumLoc: Enumeration;

		begin
			EnumLoc := EnumParVal;
			if EnumLoc = Ident3 then (* executed *)
				Func3 := true;
		end; (* Func3 *)

{$IFC UNDEFINED THINK_PASCAL}
		function TickCount: LONGINT;
		inline
			$A975;

		procedure Debugger;
		inline
			$a9ff;

		procedure ShowText;
		begin
		end;
{$ENDC}

	begin (* main program *)
(* initializations *)
		new(NextPointerGlob);
		new(PointerGlob);
		PointerGlob^.PointerComp := NextPointerGlob;
		PointerGlob^.Discr := Ident1;
		PointerGlob^.EnumComp := Ident3;
		PointerGlob^.IntComp := 40;
		PointerGlob^.StringComp := 'DHRYSTONE PROGRAM, SOME STRING';
		StringGlob1 := 'DHRYSTONE PROGRAM, 1''ST STRING';
		NOM := 0;
		NOE := 0;
		AVG := 0.0;
		MIN := 0.0;
		MAX := 0.0;
		SumTime := 0.0;
		MinTime := LargeRealNumber;
		MaxTime := 0.0;
		for MeasurementIndex := 1 to NumberOfMeasurements do
			begin
				BeginClock := TickCount; (* Start timer *)
				for ExecutionIndex := 1 to NumberOfExecutions do
					begin
						Proc5;
						Proc4;
						IntGlob1 := 2;
						IntGlob2 := 3;
						StringGlob2 := 'DHRYSTONE PROGRAMS, 2ND STRING';
						EnumGlob := Ident2;
						BoolGlob := Func2(StringGlob1, StringGlob2); (* BoolGlob = false *)
						while IntGlob1 < IntGlob2 do (* loop executed once *)
							begin
								IntGlob3 := 5 * IntGlob1 - IntGlob2; (* IntGlob3 = 7 *)
								Proc7(IntGlob1, IntGlob2, IntGlob3); (* IntGlob3 = 7 *)
								IntGlob1 := IntGlob1 + 1;
							end; (* while *)
 (* IntGlob1 = 3 *)
						Proc8(ArrayGlob1, ArrayGlob2, IntGlob1, IntGlob3); (* IntGlob = 5 *)
						Proc1(PointerGlob);
						for CharIndex := 'A' to CharGlob2 do (* executed twice *)
							if EnumGlob = Func1(CharIndex, 'C') then (* not executed *)
								Proc6(Ident1, EnumGlob); (* IntGlob1 = 3, IntGlob2 = 3, IntGlob3 = 7 *)
						IntGlob3 := IntGlob2 * IntGlob1;
						IntGlob2 := IntGlob3 div IntGlob1;
						IntGlob2 := 7 * (IntGlob3 - IntGlob2) - IntGlob1;
						Proc2(IntGlob1);
					end; (* for ExecutionIndex *)
				EndClock := TickCount;
(* Stop timer *)
				SumClocks := (EndClock - BeginClock) * MicrosecondsPerClock;
(* Measure and subtract time for NumberOfExecutions empty loops *)
				BeginClock := TickCount;
				for ExecutionIndex := 1 to NumberOfExecutions do
					begin (* empty *)
					end;
				EndClock := TickCount;
				EmptyLoopClocks := (EndClock - BeginClock) * MicrosecondsPerClock;
				SumClocks := SumClocks - EmptyLoopClocks;
				TimePerExecution := SumClocks / NumberOfExecutions;
				SumTime := SumTime + TimePerExecution;
				if TimePerExecution < MinTime then
					MinTime := TimePerExecution;
				if TimePerExecution > MaxTime then
					MaxTime := TimePerExecution;
			end; (* for MeasurementIndex *)
		NOE := NumberOfExecutions;
		NOM := NumberOfMeasurements;
		AVG := SumTime / NumberOfMeasurements;
		MIN := MinTime;
		MAX := MaxTime;
	end;

end.